/*---------------------------------------------------------------------------
 * �2005 NORSYS
 * main author : nono
 *
 * This software is a computer program whose purpose is to provide abstraction
 * for accessing directory data sources within java applications.
 *
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 *
 * Created on Nov 2, 2005
 * --------------------------------------------------------------------------*/

package speculoos.jndi.mappers;

import java.util.Map;

import javax.naming.Name;
import javax.naming.NamingException;

import speculoos.core.MapperException;
import speculoos.jndi.RenameMapper;
import speculoos.spi.MapperTechnicalException;

/**
 * Implementation for rename mapper.
 * <p>
 * Rename (or <em>move</em>) mapper changes the hierarchical structure of
 * directory by reparenting a node. This operation is implemented simply by
 * changing the <em>relative DN</em> of a node. The only parameter is the new
 * RDN.
 * </p>
 * <p>
 * <h3>Quoting from RFC2251</h3>
 * <code>
 *    The Modify DN Operation allows a client to change the leftmost (least
 significant) component of the name of an entry in the directory, or
 to move a subtree of entries to a new location in the directory.  The
 Modify DN Request is defined as follows:

 ModifyDNRequest ::= [APPLICATION 12] SEQUENCE {
 entry           LDAPDN,
 newrdn          RelativeLDAPDN,
 deleteoldrdn    BOOLEAN,
 newSuperior     [0] LDAPDN OPTIONAL }

 Parameters of the Modify DN Request are:

 - entry: the Distinguished Name of the entry to be changed.  This
 entry may or may not have subordinate entries.

 - newrdn: the RDN that will form the leftmost component of the new
 name of the entry.

 - deleteoldrdn: a boolean parameter that controls whether the old RDN
 attribute values are to be retained as attributes of the entry, or
 deleted from the entry.

 - newSuperior: if present, this is the Distinguished Name of the entry
 which becomes the immediate superior of the existing entry.

 * </code>
 * </p>
 * <p>
 * This implementation expects a variable "newRDN" in its environment that can
 * be parsed to a name.
 * </p>
 * 
 * @author nono
 * @version $Id$
 */
public class RenameMapperImpl extends JNDIMapper implements RenameMapper {

	/**
	 * @param value
	 */
	public RenameMapperImpl(String value) {
		super(value);
	}

	/**
	 * 
	 */
	public RenameMapperImpl() {
		super("rename");
	}
	
	/*
	 *  (non-Javadoc)
	 * @see speculoos.jndi.RenameMapper#rename(java.lang.Object, java.util.Map)
	 */
	public Object rename(Object input, Map env) throws MapperException {
		return map(input, env);
	}

	/*
	 *  (non-Javadoc)
	 * @see speculoos.jndi.mappers.JNDIMapper#doMap(java.lang.Object, java.util.Map)
	 */
	protected Object doMap(Object input, Map env) throws MapperException {
		String newRDN = (String) env.get("newRDN");
		/* instanciate new RDN */
		if (newRDN == null)
			throw new IllegalArgumentException(
					"The new relative name is not set");
		Name _nrdn;
		try {
			_nrdn = nameParser.parse(newRDN);
		} catch (NamingException e1) {
			throw new MapperTechnicalException(
					"Cannot resolve name for new RDN context " + newRDN);
		}
		/* do query */
		try {
			if (log.isDebugEnabled())
				log.debug("[" + getName() + "] renaming " + _root + " to "
						+ _nrdn);
			directory.rename(_root, _nrdn);
			if (log.isDebugEnabled())
				log.debug("[" + getName() + "] done renaming");
		} catch (NamingException ne) {
			throw new MapperException("Error in modificaiton operation", ne);
		}
		return input;
	}

}
